"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.validate = void 0;
function validate(key, value, spec) {
    if (value === null) {
        if (spec.nullable.value) {
            return;
        }
        throw new Error("Unexpected null value for key '" + key + "'");
    }
    switch (spec.target) {
        case "object":
            return validateObject(key, value, spec);
        case "array":
            return validateArray(key, value, spec);
        case "string":
            return validateString(key, value);
        case "number":
            return validateNumber(key, value);
        case "boolean":
            return validateBoolean(key, value);
        case "unknown":
            return;
        default:
            assertNever(spec);
    }
}
exports.validate = validate;
function validateString(key, value) {
    if (typeof value !== "string") {
        throw new Error("Expected value of key '" + key + "' to be of type 'string', but got '" + typeOf(value) + "'");
    }
}
function validateNumber(key, value) {
    if (typeof value !== "number") {
        throw new Error("Expected value of key '" + key + "' to be of type 'number', but got '" + typeOf(value) + "'");
    }
}
function validateBoolean(key, value) {
    if (typeof value !== "boolean") {
        throw new Error("Expected value of key '" + key + "' to be of type 'boolean', but got '" + typeOf(value) + "'");
    }
}
var EXACT = false;
function validateObject(key, value, spec) {
    if (typeof value !== "object") {
        throw new Error("Expected value of key '" + key + "' to be of type 'object', but got '" + typeof value + "'");
    }
    if (Array.isArray(value)) {
        throw new Error("Expected value of key '" + key + "' to be of type 'object', but got 'array'");
    }
    // null checking for type narrowing
    if (value === null) {
        throw new Error("Unexpected null value for key '" + key + "'");
    }
    var properties = Object.assign({}, value);
    for (var _i = 0, _a = Object.keys(spec.entries); _i < _a.length; _i++) {
        var _key = _a[_i];
        var _spec = spec.entries[_key];
        var _value = properties[_key];
        if (_value === undefined) {
            if (_spec.optional.value) {
                continue;
            }
            throw new Error("Missing required key '" + _key + "'");
        }
        validate(_key, _value, _spec);
        delete properties[_key];
    }
    if (Object.keys(properties).length > 0 && EXACT) {
        throw new Error("Excessive keys exist: " + Array.from(Object.keys(properties)).join(", "));
    }
}
function validateArray(key, value, spec) {
    if (!Array.isArray(value)) {
        throw new Error("Expected value of key '" + key + "' to be of type 'array', but got '" + typeof value + "'");
    }
    value.forEach(function (it, i) { return validate(key + "[" + i + "]", it, spec.element); });
}
function assertNever(x) {
    throw new Error("Unexpected runtime value: " + x);
}
function typeOf(x) {
    if (typeof x === "object") {
        return Array.isArray(x) ? "array" : "object";
    }
    return typeof x;
}
//# sourceMappingURL=validators.js.map